# Module for parsing /accessing dbvdb output
# Ed Green
# (c) 2002 / Ed Green, UC Berkeley
# $Header: /home/ed/source_control/psce/pms/DBVDB.pm,v 1.4 2005/03/27 20:52:32 ed Exp $

=head1 NAME

DBVDB.pm

=head1 SYNOPSIS

my $dbvdb_o = new DBVDB( 'dbvdb.sum' );

print ( "Number of query database sequences:",
    $dbvdb_o->numQueries(), "\n" );

print ( "Number of target database sequences:",
    $dbvdb_o->numTargets(), "\n" );

foreach my $query ( $dbvdb_o->queries() ) {
    print( "Here are the hits against $query:\n" );
    foreach my $hit ( $dbvdb_o->hitsOnQuery( $query ) ) {
        print( "\t", $dbvdb_o->hitTargetId( $hit ),
               "\t", $dbvdb_o->hitSig( $hit ), "\n" );
    }
}


=head1 DESCRIPTION

DBVDB is a module that parses a dbvdb output file, stores the
data internally, and makes the data available through the
methods below

=cut

package DBVDB;
use strict;

=head2 new

  Title:    new
  Function: constructor
  Returns:  a new DBVDB object
  Args:     a dbvdbsum.pl output file

=cut

sub new {
    my ( $class, $file ) = @_;
    my $self = &_parse( $file );
    bless( $self );
    return $self;
}


=head2 sigs

  Title:    sigs 
  Function: accessor
  Returns:  sorted array of all significance values seen (every hit
	    was at one of the significances in this list)
  Args:     none

=cut

sub sigs {
    my $self = shift;
    my @sigs = sort { $a <=> $b } keys %{ $self->{ 'hits' } };
    return @sigs;
}


=head2 hitsAtSig

  Title:    hitsAtSig
  Function: accessor
  Returns:  pointer to array of hits at this significance
  Args:     significance value (scalar)

=cut

sub hitsAtSig {
    my $self = shift;
    my $sig = shift;
    if ( defined( $self->{ 'hits' }->{ $sig } ) ) {
	return $self->{ 'hits' }->{ $sig };
    }
    else {
	return ([]);
    }
}


=head2 queries

  Title:    queries
  Function: accessor
  Returns:  array of ids to all query sequences involved in a hit
  Args:     none

=cut

sub queries {
    my $self = shift;
    return keys %{ $self->{ 'query_hits' } };
}


=head2 hitsOnQuery

  Title:    hitsOnQuery
  Function: accessor
  Returns:  array of hits to this query 
  Args:     query id

=cut

sub hitsOnQuery {
    my $self = shift;
    my $query = shift;
    if ( defined( $self->{ 'query_hits' }->{ $query } ) ) {
	return @{ $self->{ 'query_hits' }->{ $query } };
    }
    else {
	return ([]);
    }
}

=head2 targets

  Title:    targets
  Function: accessor
  Returns:  array of ids to all target sequences involved in a hit
  Args:     none

=cut

sub targets {
    my $self = shift;
    return keys %{ $self->{ 'target_hits' } };
}


=head2 hitsOnTarget

  Title:    hitsOnTarget
  Function: accessor
  Returns:  array of hits to this target 
  Args:     target id

=cut

sub hitsOnTarget {
    my $self = shift;
    my $target = shift;
    if ( defined( $self->{ 'target_hits' }->{ $target } ) ) {
	return @{ $self->{ 'target_hits' }->{ $target } };
    }
    else {
	return([]);
    }
}

=head2 queryDb

  Title:    queryDb
  Function: accessor
  Returns:  the name of the queryDb used
  Args:     none

=cut

sub queryDb {
    my $self = shift;
    return $self->{ 'qdb' };
}


=head2 targetDb

  Title:    targetDb
  Function: accessor
  Returns:  the name of the targetDb used
  Args:     none

=cut

sub targetDb {
    my $self = shift;
    return $self->{ 'tdb' };
}


=head2 numQueries

  Title:    numQueries
  Function: accessor
  Returns:  the number sequences in the query database
  Args:     none

=cut

sub numQueries {
    my $self = shift;
    return $self->{ 'num_q' };
}


=head2 numTargets

  Title:    numTargets
  Function: accessor
  Returns:  the number of sequences in the target database
  Args:     none

=cut

sub numTargets {
    my $self = shift;
    return $self->{ 'num_t' };
}


=head2 numQueryRels

  Title:    numQueryRels
  Function: accessor
  Returns:  pointer to three element list of the number of
    SCOP-defined relationships between sequences in the query
    database. The three elements are: unnormalized, linearly
    normalized, and quadratically normalized
  Args:     none

=cut

sub numQueryRels {
    my $self = shift;
    return $self->{ 'num_q_rels' };
}


=head2 numTargetRels

  Title:    numTargetRels
  Function: accessor
  Returns:  pointer to three element list of the number of
    SCOP-defined relationships between sequences in the target
    database. The three elements are: unnormalized, linearly
    normalized, and quadratically normalized
  Args:     none

=cut

sub numTargetRels {
    my $self = shift;
    return $self->{ 'num_t_rels' };
}


=head2 hitQueryId

  Title:    hitQueryId
  Function: accessor
  Returns:  id of the query sequence involved in a hit
  Args:     a hit pointer

=cut

sub hitQueryId {
    my $self  = shift;
    my $hit_p = shift;
    return $hit_p->[ 0 ];
}


=head2 hitTargetId

  Title:    hitTargetId
  Function: accessor
  Returns:  id of the target sequence involved in a hit
  Args:     a hit pointer

=cut

sub hitTargetId {
    my $self  = shift;
    my $hit_p = shift;
    return $hit_p->[ 1 ];
}


=head2 hitSig

  Title:    hitSig
  Function: accessor
  Returns:  the significance value of a hit
  Args:     a hit pointer

=cut

sub hitSig {
    my $self = shift;
    my $hit_p = shift;
    return $hit_p->[ 2 ];
}


=head2 hitRel

  Title:    hitRel
  Function: accessor
  Returns:  the SCOP-defined relatedness of a hit
    0=different class
    1=same class, different fold;
    2=same fold, different superfamily
    3=same superfamily
  Args: a hit pointer

=cut

sub hitRel {
    my $self = shift;
    my $hit_p = shift;
    return $hit_p->[ 3 ];
}


=head2 hitWeightUnnorm

  Title:    hitWeightUnnorm
  Function: accessor
  Returns:  the normalization factor under no normalization for this
  hit, i.e. 1
  Args: a hit pointer

=cut

sub hitWeightUnnorm {
    my $self = shift;
    my $hit_p = shift;
    return $hit_p->[ 4 ];
}


=head2 hitWeightLinNorm

  Title:    hitWeightLinNorm
  Function: accessor
  Returns:  the normalization factor under linear normalization for
    this hit, based on its superfamily size
  Args:     a hit pointer

=cut

sub hitWeightLinNorm {
    my $self = shift;
    my $hit_p = shift;
    return $hit_p->[ 5 ];
}


=head2 hitWeightQuadNorm

  Title:    hitWeightQuadNorm
  Function: accessor
  Returns:  the normalization factor under quadratic normalization for
    this hit, based on its superfamily size
  Args:     a hit pointer

=cut

sub hitWeightQuadNorm {
    my $self = shift;
    my $hit_p = shift;
    return $hit_p->[ 6 ];
}


=head2 _parse

  Title:    _parse
  Function: internal
  Returns:  
  Args:

=cut

sub _parse {
    my $fn = shift;
    my ( %hits, # keyed by sig
	 %queries, # keyed by id to array of hits
	 %targets, # keyed by id to array of hits
	 %self,
	 $l,
	 $comp );


    if ( $comp = &_isCompressed( $fn ) ) {
	open( FH, "$comp"."cat $fn|" ) or
	    die ( "$comp"."cat $fn: $!" );
    }
    else {
	open( FH, $fn ) or die ( "$fn: $!" );
    }


    $l = <FH>; &_checkHead( $l );
    $l = <FH>; my $qdb = &_getDb( $l );
    $l = <FH>; my $tdb = &_getDb( $l );
    $l = <FH>; my $num_q = &_getNum( $l );
    $l = <FH>; my $num_t = &_getNum( $l );
    $l = <FH>; my @num_q_rels = &_getRels( $l );
    $l = <FH>; my @num_t_rels = &_getRels( $l );
    <FH>;
    <FH>;
    while ( my $line = <FH> ) {
	my @this_hit = &_parseHit( $line );
	push( @{ $hits{ $this_hit[ 2 ] } }, \@this_hit );
	push( @{ $queries{ $this_hit[ 0 ] } }, \@this_hit );
	push( @{ $targets{ $this_hit[ 1 ] } }, \@this_hit );
    }
    $self{ 'qdb' } = $qdb;
    $self{ 'tdb' } = $tdb;
    $self{ 'num_q' } = $num_q;
    $self{ 'num_t' } = $num_t;
    $self{ 'num_q_rels' } = \@num_q_rels;
    $self{ 'num_t_rels' } = \@num_t_rels;
    $self{ 'hits' } = \%hits;
    $self{ 'query_hits' } = \%queries;
    $self{ 'target_hits' } = \%targets;
    return \%self;
}

sub _checkHead {
    my $head = shift;
    unless( $head =~ /dbvdbsum v\d+\.\d+ results/ ) {
	die( "doesn't look like a dbvdb output file to me" );
    }
}

sub _getDb {
    my $db_line = shift;
    my ($db) = ($db_line =~ /\S+ database: (.+)/);
    return $db;
}

sub _getNum {
    my $num_line = shift;
    my ($num) = ($num_line =~ /\S+ database size: (\d+)/);
    return $num;
}

sub _getRels {
    my $rel_line = shift;
    my (@rel) = split( /\s+/, $rel_line );
    return ( @rel[ 3..5 ] );
}

sub _parseHit {
    my $hit_line = shift;
    my (@el) =
	($hit_line =~/(\S+) (\S+) (\S+) (\S+) (\S+) (\S+) (\S+)/);
    return @el;
}

sub _isCompressed {
    my $fn = shift;
    if ( substr( $fn, length( $fn ) - 2 ) eq 'gz' ) {
	return 'gz';
    }
    elsif ( substr( $fn, length( $fn ) - 3 ) eq 'bz2' ) {
	return 'bz';
    }
    else {
	return 0;
    }
}    

1;
